@@ -3,249 +3,10 @@
 
 XBase - Perl module for reading and writing the dbf files
 
-=head1 SYNOPSIS
-
-	use XBase;
-	my $table = new XBase("dbase.dbf") or die XBase->errstr();
-	for (0 .. $table->last_record())
-		{
-		my ($deleted, $id, $msg)
-			= $table->get_record($_, "ID", "MSG");
-		print "$id:\t$msg\n" unless $deleted;
-		}
-
-=head1 DESCRIPTION
-
-This module can read and write XBase database file, known as dbf in
-dBase and FoxPro world. It also reads memo fields from the dbt files,
-if needed. Module XBase provides simple native interface to XBase
-files and you can also use the DBD::XBase DBI driver (is not complete
-yet).
-
-B<Warning> for now: It doesn't support any index files at the present
-time! That means if you change your dbf, your idx&mdx (if you have
-any) will not match. So do not do that.
-
-The following methods are supported:
-
-=head2 General methods
-
-=over 4
-
-=item new
-
-Creates the XBase object, the argument gives the dbf file (table in
-fact) name. A suffix .dbf will be appended if needed. This method
-creates a new object and reads the file header. It will also read
-appropriate memo file if there is a memo field in dbf.
-
-=item close
-
-Closes the object/file.
-
-=item create
-
-Creates new database file on disk and initializes it with 0 records.
-A dbt (memo) file will be also created if the table contains some memo
-field. Arguments to create are passed as hash.
-
-You can call this method as method of another XBase object and then
-you only need to pass B<name> value of the hash; the structure
-(fields) of the new file will be the same as of the original object.
-
-If you call create using class name (XBase), you have to (besides
-B<name>) also specify another four values, each being a reference
-to list: B<field_names>, B<field_types>, B<field_lengths> and
-B<field_decimals>. The field types are specified by one letter
-strings. If you set certain value (except field name) as undefined,
-create will make it into some reasonable default.
-
-The new file mustn't exist yet -- XBase will not allow you to
-overwrite existing table. Use B<drop> to delete them first (or unlink).
-
-=item drop
-
-This method closes the table and deletes it on disk (including dbt
-file, if there is any).
-
-=item last_record
-
-Returns number of the last record in the file. The lines deleted but
-present in the file are included in this number.
-
-=item last_field
-
-Returns number of the last field in the file, number of fields minus 1.
-
-=item field_names, field_types, field_lengths, field_decimals
-
-Return list of field names and so on for the dbf file.
-
-=back
-
-When dealing with the records, reading or writing, you always have
-to specify the number of the record in the file. The range is
-0 .. $table->last_record().
-
-=head2 Reading the data
-
-=over 4
-
-=item get_record
-
-Returns a list of data (field values) from the specified record (line
-of the table). The first argument in the call is the number of the
-record. If you do not specify any other argument, all fields are
-returned. You can also put list of field names after the record number
-and then only those will be returned. The first value of the returned
-list is the 1/0 _DELETED value saying if the record is deleted or not,
-so on success, get_record will never return empty list.
-
-=item get_record_as_hash
-
-Returns hash (in list context) or reference to hash (in scalar
-context) containing field values indexed by field names. The deleted
-flag has name _DELETED. The only argument in the call is the record
-number.
-
-=back
-
-=head2 Writing the data
-
-=over 4
-
-=item set_record
-
-As arguments, takes the number of the record and the list of values
-of the fields. It writes the record to the file. Unspecified fields
-(if you pass less than you should) are set to undef/empty. The record
-is undeleted.
-
-=item set_record_hash
-
-Takes number of the record and hash as arguments, sets the fields,
-unspecified are undeffed/emptied. The record is undeleted.
-
-=item update_record_hash
-
-Like B<write_record_hash> but preserves fields that do not have value
-specified in the hash. The record is undeleted.
-
-=item delete_record, undelete record
-
-Deletes/undeletes the record.
-
-=back
-
-The set and update methods return the record number actually written,
-number 0 is returned as true.
-
-=head2 Errors and debugging
-
-If the method fails (returns undef of null list), the error message
-can be retrieved via B<errstr> method. If the B<new> or B<create>
-method fails, you have no object so you get the error message using
-class syntax XBase->errstr().
-
-The methods B<get_header_info> returns (not prints) information about
-the file and about the fields, B<dump_records> prints all records from
-the file, one on a line, fields separated by commas.
-
-Module XBase::Base(3) defines some basic functionality and also following
-variables, that affect the internal behaviour:
-
-=over 4
-
-=item $DEBUG
-
-Enables error messages on stderr.
-
-=item $FIXPROBLEMS
-
-When reading the file, try to continue, even if there is some
-(minor) missmatch in the data.
-
-=back
-
-In the module XBase there is variable $CLEARNULLS that specifies,
-whether will the reading methods cuts off spaces and nulls from the
-end of fixed character fields on read. The default is true.
-
-=head1 LITTLE EXAMPLE
-
-This is a code to update field MSG in record where ID is 123.
-
-	use XBase;
-	my $table = new XBase("test.dbf") or die XBase->errstr();
-	for (0 .. $table->last_record())
-		{
-		my ($deleted, $id)
-			= $table->get_record($_, "ID")
-		die $table->errstr unless defined $deleted;
-		next if $deleted;
-		if ($id == 123)
-			{
-			$table->update_record_hash($_,
-				"MSG" => "New message");
-			last;
-			}
-		}
-
-=head1 MEMO FIELDS and INDEX FILES
-
-If there is a memo field in the dbf file, the module tries to open
-file with the same name but extension .dbt. It uses module
-XBase::Memo(3) for this. It reads and writes this memo field
-transparently (ie you do not know about it).
-
-No index files are currently supported. Two reasons: you do not need
-them when reading the file because you specify the record number
-anyway and writing them is extremely difficult. I will try to add the
-support but do not promise anything ;-) There are too many too complex
-questions: how about compound indexes? Which index formats should
-I support? What files contain the index data? I do not have dBase nor
-Fox* so do not have data to experiment.
-
-Please send me examples of your data files and suggestions for
-interface.
-
-=head1 HISTORY
-
-I have been using the Xbase(3) module by Pratap Pereira for quite
-a time to read the dbf files, but it had no writing capabilities, it
-was not C<use strict> clean and the author did not support the
-module behind the version 1.07. So I started to make my own patches
-and thought it would be nice if other people could make use of them.
-I thought about taking over the development of the original Xbase
-package, but the interface seemed rather complicated to me and I also
-disliked the licence Pratap had about the module.
-
-So with the help of article XBase File Format Description by Erik
-Bachmann on URL http://www.geocities.com/SiliconValley/Pines/2563/xbase.htm
-I have written a new module. It doesn't use any code from Xbase-1.07
-and you are free to use and distribute it under the same terms as Perl
-itself.
-
-Please send all bug reports cc'ed to my e-mail, since I might miss
-your post in c.l.p.misc or dbi-users (or other groups). Any comments
-both about the Perl and XBase are welcome.
-
-=head1 VERSION
-
-0.039
-
-=head1 AUTHOR
-
-(c) Jan Pazdziora, adelton@fi.muni.cz
-
-=head1 SEE ALSO
-
-perl(1); DBD::XBase(3) and DBI(3) for DBI interface
-
 =cut
 
 # ########
-use 5.004;	# Yes, 5.004 and everything should be fine
+use 5.004;		# Yes, 5.004 and everything should be fine
 
 # #############################
 # Here starts the XBase package
@@ -263,7 +24,7 @@ use vars qw( $VERSION $errstr $CLEARNULLS @ISA );
 
 @ISA = qw( XBase::Base );
 
-$VERSION = "0.039";
+$VERSION = "0.058";
 
 $errstr = "Use of \$XBase::errstr is depreciated, please use XBase->errstr() instead\n";
 
@@ -273,28 +34,18 @@ $CLEARNULLS = 1;
 # ########################
 # Constructor, open, close
 
-# Constructor of the class; expects class name and filename of the
-# .dbf file, returns the object if the file can be read, null otherwise
-sub new
-	{
-	my $class = shift;
-	my $result = $class->SUPER::new(@_);
-	###
-	### depreciated
-	### $errstr = $XBase::Base::errstr unless $result;
-	###
-	$result;
-	}
+# Open the specified file or try to append .dbf suffix.
 sub open
 	{
 	my $self = shift;
-	if (not defined $self->{'opened'} or not $self->{'opened'})
+	if (@_ and not -f $_[0])
 		{
-		if (@_ and not defined $self->{'filename'})
-			{ $self->{'filename'} = shift; }
-		my $filename = $self->{'filename'};
-		if ((not -f $filename) and $filename !~ /\.dbf$/i)
-			{ $self->{'filename'} = $filename . '.dbf'; }
+		my $filename = shift;
+		if (-f ($filename . '.dbf'))
+			{ $filename .= '.dbf'; }
+		elsif (-f ($filename . '.DBF'))
+			{ $filename .= '.DBF'; }
+		unshift @_, $filename;
 		}
 	$self->SUPER::open(@_);
 	}
@@ -311,7 +62,7 @@ sub read_header
 	my ($version, $last_update, $num_rec, $header_len, $record_len,
 		$res1, $incompl_trans, $enc_flag, $rec_thread,
 		$multiuser, $mdx_flag, $language_dr, $res2)
-		= unpack "Ca3Vvva2CCVa8CCa2", $header;
+		= unpack 'Ca3Vvva2CCVa8CCa2', $header;
 				# parse the data
 
 	my ($names, $types, $lengths, $decimals) = ( [], [], [], [] );
@@ -333,8 +84,9 @@ sub read_header
 		my ($name, $type, $address, $length, $decimal,
 			$multiuser1, $work_area, $multiuser2,
 			$set_fields_flag, $res, $index_flag)
-				= unpack "A11aVCCa2Ca2Ca7C", $field_def;
-	
+				= unpack 'A11aVCCa2Ca2Ca7C', $field_def;
+
+		$name =~ s/[\000 ].*$//s;
 		$name = uc $name;
 
 		if ($type eq "C")
@@ -371,10 +123,18 @@ sub init_memo_field
 	{
 	my $self = shift;
 	return $self->{'memo'} if defined $self->{'memo'};
-	my $dbtname = $self->{'filename'};
-	$dbtname =~ s/(\.dbf)?$/.dbt/i;
 	require XBase::Memo;
-	return XBase::Memo->new($dbtname);
+
+	my $memoname = $self->{'filename'};
+	$memoname =~ s/\.DBF?$/.DBT/;	$memoname =~ s/(\.dbf)?$/.dbt/;
+	my $memo = return XBase::Memo->new($memoname, $self->{'version'});
+	if (not defined $memo)
+		{
+		$memoname = $self->{'filename'};
+		$memoname =~ s/\.DBF?$/.FPT/;	$memoname =~ s/(\.dbf)?$/.fpt/;
+		$memo = return XBase::Memo->new($memoname, $self->{'version'});
+		}
+	$memo;
 	}
 
 sub close
@@ -402,16 +162,37 @@ sub field_decimals	{ @{shift->{'field_decimals'}}; }
 # Return field number for field name
 sub field_name_to_num
 	{
-	my $self = shift;
-	my $name = shift;
+	my ($self, $name) = @_;
 	$self->{'hash_names'}{$name};
 	}
 
+sub field_type
+	{
+	my ($self, $name) = @_;
+	my $num = $self->field_name_to_num($name);
+	return unless defined $num;
+	($self->field_types)[$num];
+	}
+sub field_length
+	{
+	my ($self, $name) = @_;
+	my $num = $self->field_name_to_num($name);
+	return unless defined $num;
+	($self->field_lengths)[$num];
+	}
+sub field_decimal
+	{
+	my ($self, $name) = @_;
+	my $num = $self->field_name_to_num($name);
+	return unless defined $num;
+	($self->field_decimals)[$num];
+	}
 
 # #############################
 # Header, field and record info
 
 # Returns (not prints!) the info about the header of the object
+*header_info = \&get_header_info;
 sub get_header_info
 	{
 	my $self = shift;
@@ -438,7 +219,7 @@ EOF
 sub get_field_info
 	{
 	my ($self, $num) = @_;
-	sprintf "%d.\t%-16.16s%-8.8s%-8.8s%-8.8s\n", $num + 1,
+	sprintf "%d.\t%-16.16s%-8.8s%-8.8s%s\n", $num + 1,
 		map { $self->{$_}[$num] }
 			qw( field_names field_types field_lengths field_decimals );
 	}
@@ -477,10 +258,10 @@ sub decode_version_info
 		}
 	
 	my $result = "ver. $vbits";
-	if ($dbtflag)
+	if ($memo)
+		{ $result .= " with memo file"; }
+	elsif ($dbtflag)
 		{ $result .= " with DBT file"; }
-	elsif ($memo)
-		{ $result .= " with some memo file"; }
 	$result .= " containing SQL table" if $sqltable;
 	$result;
 	}
@@ -498,8 +279,8 @@ sub decode_version_info
 
 sub get_record
 	{
-	NullError();
 	my ($self, $num, @fields) = @_;
+	$self->NullError();
 
 	my @data = $self->read_record($num);
 				# SUPER will uncache/unpack for us
@@ -568,10 +349,8 @@ sub process_list_on_read
 			my $len = $self->{'field_lengths'}[$num - 1];
 			my $dec = $self->{'field_decimals'}[$num - 1];
 			if ($dec)
-				{ $data[$num] = sprintf "%-$len.${dec}f", $value; }
-			else
-				{ $data[$num] = sprintf "%-${len}d", $value; }
-			$data[$num] += 0;
+				{ substr($value, -$dec, 0) = '.'; }
+			$data[$num] = $value + 0;
 			}
 		elsif ($type =~ /^[MGBP]$/)
 			{
@@ -592,8 +371,8 @@ sub process_list_on_read
 # Record is always undeleted
 sub set_record
 	{
-	NullError();
 	my ($self, $num) = (shift, shift);
+	$self->NullError();
 	my @data = $self->process_list_on_write($num, @_,
 				(undef) x ($self->last_field - $#_));
 	$self->write_record($num, " ", @data);
@@ -605,8 +384,8 @@ sub set_record
 # undef/empty
 sub set_record_hash
 	{
-	NullError();
 	my ($self, $num, %data) = @_;
+	$self->NullError();
 	$self->set_record($num, map { $data{$_} } @{$self->{'field_names'}} );
 	}
 
@@ -614,8 +393,8 @@ sub set_record_hash
 # unchanged
 sub update_record_hash
 	{
-	NullError();
 	my ($self, $num, %data) = @_;
+	$self->NullError();
 
 	my @data = $self->get_record($num);	# read the original data first
 	return unless @data;
@@ -651,19 +430,19 @@ sub write_record
 # Delete and undelete record
 sub delete_record
 	{
-	NullError();
 	my ($self, $num) = @_;
+	$self->NullError();
 	if ($num > $self->last_record())
-		{ Error "Can't delete record number $num, there is not so many of them\n"; return;}
+		{ $self->Error("Can't delete record number $num, there is not so many of them\n"); return;}
 	$self->write_record($num, "*");
 	1;
 	}
 sub undelete_record
 	{
-	NullError();
 	my ($self, $num) = @_;
+	$self->NullError();
 	if ($num > $self->last_record())
-		{ Error "Can't undelete record number $num, there is not so many of them\n"; return;}
+		{ $self->Error("Can't undelete record number $num, there is not so many of them\n"); return;}
 	$self->write_record($num, " ");
 	1;
 	}
@@ -702,8 +481,14 @@ sub process_list_on_write
 		elsif ($type =~ /^[NFD]$/)
 			{
 			$value += 0;
-			$value = sprintf "%$length.${decimal}f", $value;
-			$value =~ s/[.,]//;
+			if ($decimal)
+				{
+				$length++;
+				$value = sprintf "%$length.${decimal}f", $value;
+				$value =~ s/[.,]//;
+				}
+			else
+				{ $value = sprintf "%$length.0f", $value; }
 			}
 		elsif ($type =~ /^[MGBP]$/)
 			{
@@ -722,7 +507,8 @@ sub process_list_on_write
 				# choose another location	
 			
 				$memo_index = $self->{'memo'}
-					->write_record($memo_index, $value);
+					->write_record($memo_index,
+							$type, $value);
 				$value = $memo_index + 0;
 				}
 			else
@@ -850,7 +636,7 @@ sub drop
 	if (ref $self)
 		{
 		if (defined $self->{'memo'})
-			{ $self->{'memo'}->drop(); }
+			{ $self->{'memo'}->drop(); delete $self->{'memo'}; }
 		return $self->SUPER::drop();
 		}
 	XBase::Base::drop($filename);
@@ -858,3 +644,261 @@ sub drop
 
 1;
 
+__END__
+
+=head1 SYNOPSIS
+
+  use XBase;
+  my $table = new XBase("dbase.dbf") or die XBase->errstr();
+  for (0 .. $table->last_record())
+	{
+	my ($deleted, $id, $msg)
+		= $table->get_record($_, "ID", "MSG");
+	print "$id:\t$msg\n" unless $deleted;
+	}
+
+=head1 DESCRIPTION
+
+This module can read and write XBase database file, known as dbf in
+dBase and FoxPro world. It also reads memo fields from the dbt and fpt
+files, if needed. Module XBase provides simple native interface to
+XBase files. For DBI compliant database access, check DBD::XBase and
+DBI module.
+
+B<Warning> for now: It doesn't support any index files at the present
+time! That means if you change your dbf, your idx/mdx (if you have
+any) will not match. You will need to regenerate them using other
+tools -- probably those that later make use of them.
+
+The following methods are supported by XBase module:
+
+=head2 General methods
+
+=over 4
+
+=item new
+
+Creates the XBase object, the argument gives the name of the dbf file
+(table, in fact). A suffix .dbf will be appended if needed. This method
+creates and initializes new object, will also check for memo file, if
+needed.
+
+=item close
+
+Closes the object/file.
+
+=item create
+
+Creates new database file on disk and initializes it with 0 records.
+A dbt (memo) file will be also created if the table contains some memo
+fields. Arguments to create are passed as hash.
+
+You can call this method as method of another XBase object and then
+you only need to pass B<name> value of the hash; the structure
+(fields) of the new file will be the same as of the original object.
+
+If you call create using class name (XBase), you have to (besides
+B<name>) also specify another four values, each being a reference
+to list: B<field_names>, B<field_types>, B<field_lengths> and
+B<field_decimals>. The field types are specified by one letter
+strings. If you set some value as undefined, create will make it into
+some reasonable default.
+
+The new file mustn't exist yet -- XBase will not allow you to
+overwrite existing table. Use B<drop> to delete them first (or unlink).
+
+=item drop
+
+This method closes the table and deletes it on disk (including dbt
+file, if there is any).
+
+=item last_record
+
+Returns number of the last record in the file. The lines deleted but
+present in the file are included in this number.
+
+=item last_field
+
+Returns number of the last field in the file, number of fields minus 1.
+
+=item field_names, field_types, field_lengths, field_decimals
+
+Return list of field names and so on for the dbf file.
+
+=item field_type, field_length, field_decimal
+
+For a field name, returns the appropriate value. Returns undef if the
+field doesn't exist in the table.
+
+=back
+
+=head2 Reading the data
+
+When dealing with the records, reading or writing, you always have
+to specify the number of the record in the file. The range is
+0 .. $table->last_record().
+
+=over 4
+
+=item get_record
+
+Returns a list of data (field values) from the specified record (line
+of the table). The first argument in the call is the number of the
+record. If you do not specify any other arguments, all fields are
+returned in the same order as they appear in the file.
+
+You can also put list of field names after the record number and then
+only those will be returned. The first value of the returned list is
+always the 1/0 _DELETED value saying if the record is deleted or not,
+so on success, get_record will never return empty list.
+
+=item get_record_as_hash
+
+Returns hash (in list context) or reference to hash (in scalar
+context) containing field values indexed by field names. The name of
+the deleted flag is _DELETED. The only argument in the call is the
+record number.
+
+=back
+
+=head2 Writing the data
+
+All three writing methods always undelete the record. On success they
+return true -- the record number actually written.
+
+=over 4
+
+=item set_record
+
+As arguments, takes the number of the record and the list of values
+of the fields. It writes the record to the file. Unspecified fields
+(if you pass less than you should) are set to undef/empty.
+
+=item set_record_hash
+
+Takes number of the record and hash as arguments, sets the fields,
+unspecified are undeffed/emptied.
+
+=item update_record_hash
+
+Like B<set_record_hash> but fields that do not have value specified
+in the hash retain their value.
+
+=item delete_record, undelete record
+
+Deletes/undeletes specified record.
+
+=back
+
+=head2 Errors and debugging
+
+If the method fails (returns false or null list), the error message
+can be retrieved via B<errstr> method. If the B<new> or B<create>
+method fails, you have no object so you get the error message using
+class syntax XBase->errstr().
+
+The methods B<get_header_info> returns (not prints) string with
+information about the file and about the fields. Method B<dump_records>
+prints all records from the file, one on a line, fields separated by
+commas.
+
+Module XBase::Base(3) defines some basic functionality and also following
+variables, that affect the internal behaviour:
+
+=over 4
+
+=item $DEBUG
+
+Enables error messages on stderr, zero by default.
+
+=item $FIXPROBLEMS
+
+When reading the file, try to continue, even if there is some
+(minor) missmatch in the data, true by default.
+
+=back
+
+In the module XBase there is variable $CLEARNULLS that specifies,
+whether will the reading methods cut off spaces and nulls from the
+end of fixed character fields on read. The default is true.
+
+=head1 LITTLE EXAMPLE
+
+This is a code to update field MSG in record where ID is 123.
+
+    use XBase;
+    my $table = new XBase("test.dbf") or die XBase->errstr();
+    for (0 .. $table->last_record())
+    	{
+    	my ($deleted, $id)
+    		= $table->get_record($_, "ID")
+    	die $table->errstr unless defined $deleted;
+    	next if $deleted;
+    	if ($id == 123)
+    		{
+    		$table->update_record_hash($_,
+    			"MSG" => "New message");
+    		last;
+    		}
+    	}
+
+Some more examples are in the eg directory of the distribution.
+
+=head1 MEMO FIELDS and INDEX FILES
+
+If there is a memo field in the dbf file, the module tries to open
+file with the same name but extension dbt or fpt. It uses module
+XBase::Memo(3) for this. It reads and writes this memo field
+transparently (you do not know about it).
+
+No index files are currently supported. Two reasons: you do not need
+them when reading the file because you specify the record number
+anyway and writing them is extremely difficult. I might try to add the
+support but do not promise anything ;-) There are too many too complex
+questions: how about compound indexes? Which index formats should
+I support? What files contain the index data? I do not have dBase nor
+Fox* so do not have data to experiment.
+
+Please send me examples of your data files and suggestions for
+interface if you need indexes.
+
+=head1 HISTORY
+
+I have been using the Xbase(3) module by Pratap Pereira for quite
+a time to read the dbf files, but it had no writing capabilities, it
+was not C<use strict> clean and the author did not support the
+module behind the version 1.07. So I started to make my own patches
+and thought it would be nice if other people could make use of them.
+I thought about taking over the development of the original Xbase
+package, but the interface seemed rather complicated to me.
+
+So with the help of article XBase File Format Description by Erik
+Bachmann on URL
+
+ http://www.geocities.com/SiliconValley/Pines/2563/xbase.htm
+
+I have written a new module. It doesn't use any code from Xbase-1.07
+and you are free to use and distribute it under the same terms as Perl
+itself.
+
+Please send all bug reports or patches CC'ed to my e-mail, since I
+might miss your post in c.l.p.misc or dbi-users (or other groups). Any
+comments about both the Perl and XBase issues of this module are also
+welcome.
+
+=head1 VERSION
+
+0.058
+
+=head1 AUTHOR
+
+(c) Jan Pazdziora, adelton@fi.muni.cz, http://www.fi.muni.cz/~adelton/
+
+at Faculty of Informatics, Masaryk University in Brno, Czech Republic
+
+=head1 SEE ALSO
+
+perl(1); DBD::XBase(3) and DBI(3) for DBI interface
+
+=cut
+
